@@ -17,6 +17,7 @@ eval 'require Local::Example::Plain;  1' or warn $@;
 eval 'require Local::Example::Marlin; 1' or warn $@;
 eval 'require Local::Example::Moo;    1' or warn $@;
 eval 'require Local::Example::Moose;  1' or warn $@;
+eval 'require Local::Example::Mouse;  1' or warn $@;
 eval 'require Local::Example::Tiny;   1' or warn $@;
 
 my ( %constructors, %accessors, %delegations, %combined );
@@ -89,37 +90,41 @@ say "";
 
 __END__
 [[ CONSTRUCTORS ]]
-         Rate   Tiny  Plain    Moo  Moose Marlin   Core
-Tiny   1317/s     --    -2%   -48%   -53%   -54%   -72%
-Plain  1340/s     2%     --   -47%   -53%   -53%   -72%
-Moo    2527/s    92%    89%     --   -11%   -12%   -47%
-Moose  2828/s   115%   111%    12%     --    -2%   -40%
-Marlin 2873/s   118%   114%    14%     2%     --   -39%
-Core   4727/s   259%   253%    87%    67%    65%     --
+         Rate  Plain   Tiny    Moo  Moose   Core Marlin  Mouse
+Plain  1350/s     --    -1%   -46%   -55%   -73%   -73%   -78%
+Tiny   1369/s     1%     --   -45%   -54%   -72%   -72%   -78%
+Moo    2495/s    85%    82%     --   -17%   -50%   -50%   -60%
+Moose  3001/s   122%   119%    20%     --   -40%   -40%   -52%
+Core   4974/s   268%   263%    99%    66%     --    -0%   -20%
+Marlin 4979/s   269%   264%   100%    66%     0%     --   -20%
+Mouse  6218/s   360%   354%   149%   107%    25%    25%     --
 
 [[ ACCESSORS ]]
-          Rate   Tiny  Moose  Plain   Core    Moo Marlin
-Tiny   17345/s     --    -1%    -3%    -7%   -36%   -45%
-Moose  17602/s     1%     --    -2%    -6%   -35%   -44%
-Plain  17893/s     3%     2%     --    -4%   -34%   -44%
-Core   18732/s     8%     6%     5%     --   -31%   -41%
-Moo    27226/s    57%    55%    52%    45%     --   -14%
-Marlin 31688/s    83%    80%    77%    69%    16%     --
+          Rate   Core   Tiny  Moose  Plain    Moo Marlin  Mouse
+Core   19907/s     --    -8%    -9%   -10%   -43%   -54%   -57%
+Tiny   21561/s     8%     --    -2%    -3%   -38%   -50%   -53%
+Moose  21913/s    10%     2%     --    -1%   -37%   -50%   -53%
+Plain  22119/s    11%     3%     1%     --   -36%   -49%   -52%
+Moo    34662/s    74%    61%    58%    57%     --   -20%   -25%
+Marlin 43555/s   119%   102%    99%    97%    26%     --    -6%
+Mouse  46291/s   133%   115%   111%   109%    34%     6%     --
 
 [[ DELEGATIONS ]]
-         Rate   Tiny   Core  Plain  Moose    Moo Marlin
-Tiny    675/s     --   -56%   -57%   -59%   -61%   -61%
-Core   1518/s   125%     --    -4%    -8%   -13%   -13%
-Plain  1581/s   134%     4%     --    -4%    -9%   -10%
-Moose  1642/s   143%     8%     4%     --    -5%    -6%
-Moo    1736/s   157%    14%    10%     6%     --    -1%
-Marlin 1752/s   160%    15%    11%     7%     1%     --
+         Rate   Tiny  Mouse   Core  Plain  Moose    Moo Marlin
+Tiny    822/s     --   -34%   -56%   -57%   -59%   -62%   -63%
+Mouse  1247/s    52%     --   -33%   -35%   -38%   -42%   -43%
+Core   1855/s   126%    49%     --    -3%    -8%   -14%   -15%
+Plain  1913/s   133%    53%     3%     --    -5%   -11%   -13%
+Moose  2021/s   146%    62%     9%     6%     --    -6%    -8%
+Moo    2149/s   162%    72%    16%    12%     6%     --    -2%
+Marlin 2193/s   167%    76%    18%    15%     9%     2%     --
 
 [[ COMBINED ]]
-         Rate   Tiny  Plain   Core  Moose    Moo Marlin
-Tiny    545/s     --   -48%   -56%   -58%   -60%   -64%
-Plain  1051/s    93%     --   -16%   -19%   -22%   -31%
-Core   1249/s   129%    19%     --    -4%    -8%   -18%
-Moose  1304/s   139%    24%     4%     --    -4%   -14%
-Moo    1355/s   148%    29%     8%     4%     --   -11%
-Marlin 1519/s   179%    45%    22%    17%    12%     --
\ No newline at end of file
+         Rate   Tiny  Mouse  Plain  Moose   Core    Moo Marlin
+Tiny    677/s     --   -41%   -48%   -57%   -57%   -59%   -64%
+Mouse  1151/s    70%     --   -11%   -27%   -27%   -31%   -39%
+Plain  1292/s    91%    12%     --   -18%   -18%   -22%   -31%
+Moose  1575/s   133%    37%    22%     --    -0%    -5%   -16%
+Core   1577/s   133%    37%    22%     0%     --    -5%   -16%
+Moo    1662/s   146%    44%    29%     6%     5%     --   -12%
+Marlin 1879/s   178%    63%    45%    19%    19%    13%     --
